@@ -1,6 +1,6 @@
 package Google::RestApi::DriveApi3;
 
-our $VERSION = '0.8';
+our $VERSION = '0.9';
 
 use Google::RestApi::Setup;
 
@@ -15,7 +15,7 @@ use constant {
 sub new {
   my $class = shift;
   state $check = compile_named(
-    api      => HasMethods['api'],
+    api      => HasApi,
     endpoint => Str, { default => Drive_Endpoint },
   );
   return bless $check->(@_), $class;
