@@ -1,10 +1,9 @@
 package Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator;
 
-our $VERSION = '0.8';
+our $VERSION = '0.9';
 
 use Google::RestApi::Setup;
 
-use Carp qw(cluck);
 use parent qw(Google::RestApi::SheetsApi4::RangeGroup::Iterator);
 
 sub new {
@@ -17,14 +16,18 @@ sub new {
 
   my $ptied = delete $p->{tied};
   my $tied = tied(%$ptied);
-  my $ranges = $tied->ranges();
-  my $range_group = $tied->spreadsheet()->range_group(values %$ranges);
 
+  my $ranges = $tied->ranges();
+  my @keys = keys %$ranges;
+  my @values = values %$ranges;
+
+  my $range_group = $tied->spreadsheet()->range_group(@values);
   my $self = $class->SUPER::new(
     %$p,
     range_group => $range_group,
   );
-  $self->{keys} = [ keys %$ranges ];
+
+  $self->{keys} = \@keys;
   $self->{tied} = $ptied;
 
   return bless $self, $class;
@@ -32,8 +35,9 @@ sub new {
 
 sub iterate {
   my $self = shift;
-  my $iterate = $self->SUPER::iterate(@_) or return;
-  my @ranges = $iterate->ranges();
+  my $range_group = $self->SUPER::iterate(@_) or return;
+
+  my @ranges = $range_group->ranges();
   my %ranges = map {
     $self->{keys}->[$_] => $ranges[$_];
   } (0..$#ranges);
